# add modules folder to interpreter path
import sys
import os
modules_path = '../../modules/'
if os.path.exists(modules_path):
    sys.path.insert(1, modules_path)

# init logging to file
import logging
LOG_FORMAT = ('%(levelname) -5s %(asctime)s %(name)s:%(funcName) -35s %(lineno) -5d:  %(message)s')
logging.basicConfig(
    filename='error.log', 
    level=logging.INFO, format=LOG_FORMAT)
LOGGER = logging.getLogger(__name__)

import connexion
from multiprocessing import Process
from messaging.MessageReceiver import MessageReceiver
from messaging.MessageSender import MessageSender

RABBIT_MQ_DNS_NAME = 'rabbit-mq'
RABBIT_MQ_PORT = '5672'

message_sender = None

# init message handler
def message_received_callback(channel, method, properties, body):
    print(f"### Received: {body}")

def pika_error_callback(error):
    print("Rabbit MQ error!")
    global flask_server
    flask_server.terminate()
    sys.exit()

# load swagger config 
app = connexion.App(__name__, specification_dir='configs/')
app.add_api('swagger.yml')

@app.route('/', methods=['GET'])
def api_root():
    return 'Endpoint of SMART RESTful API Gateway!'

def run_flask():
    app.run(host='0.0.0.0', port=5000, debug=True, use_reloader=False) # disable reloader so only subscribed once to rabbitmq

# start app
if __name__ == '__main__':
    message_rec = MessageReceiver(
        rabbit_mq_ip=RABBIT_MQ_DNS_NAME, rabbit_mq_port=RABBIT_MQ_PORT, 
        exchange_name='rest-gateway', exchange_type='direct', queue_name='rest-gateway', auto_ack=True)
    message_rec.start(message_received_callback, pika_error_callback)

    global flask_server
    flask_server = Process(target=run_flask)
    flask_server.start()

    flask_server.join()